"""
Plugin Manager
--------------

A plugin manager class is used to load plugins, manage the list of
loaded plugins, and proxy calls to those plugins.

The plugin managers provided with nose are:

:class:`PluginManager`
    This manager doesn't implement loadPlugins, so it can only work
    with a static list of plugins.

:class:`BuiltinPluginManager`
    This manager loads plugins referenced in ``nose.plugins.builtin``.

:class:`EntryPointPluginManager`
    This manager uses setuptools entrypoints to load plugins.

:class:`ExtraPluginsPluginManager`
    This manager loads extra plugins specified with the keyword
    `addplugins`.

:class:`DefaultPluginMananger`
    This is the manager class that will be used by default. If
    setuptools is installed, it is a subclass of
    :class:`EntryPointPluginManager` and :class:`BuiltinPluginManager`;
    otherwise, an alias to :class:`BuiltinPluginManager`.

:class:`RestrictedPluginManager`
    This manager is for use in test runs where some plugin calls are
    not available, such as runs started with ``python setup.py test``,
    where the test runner is the default unittest :class:`TextTestRunner`. It
    is a subclass of :class:`DefaultPluginManager`.

Writing a plugin manager
========================

If you want to load plugins via some other means, you can write a
plugin manager and pass an instance of your plugin manager class when
instantiating the :class:`nose.config.Config` instance that you pass to
:class:`TestProgram` (or :func:`main` or :func:`run`).

To implement your plugin loading scheme, implement ``loadPlugins()``,
and in that method, call ``addPlugin()`` with an instance of each plugin
you wish to make available. Make sure to call
``super(self).loadPlugins()`` as well if have subclassed a manager
other than ``PluginManager``.

"""
import inspect
import logging
import os
import sys
from itertools import chain as iterchain
from warnings import warn
import nose.config
from nose.failure import Failure
from nose.plugins.base import IPluginInterface
from nose.pyversion import sort_list

try:
    import pickle as pickle
except:
    import pickle
try:
    from io import StringIO
except:
    from io import StringIO


__all__ = ['DefaultPluginManager', 'PluginManager', 'EntryPointPluginManager',
           'BuiltinPluginManager', 'RestrictedPluginManager']

log = logging.getLogger(__name__)


class PluginProxy(object):
    """Proxy for plugin calls. Essentially a closure bound to the
    given call and plugin list.

    The plugin proxy also must be bound to a particular plugin
    interface specification, so that it knows what calls are available
    and any special handling that is required for each call.
    """
    interface = IPluginInterface
    def __init__(self, call, plugins):
        try:
            self.method = getattr(self.interface, call)
        except AttributeError:
            raise AttributeError("%s is not a valid %s method"
                                 % (call, self.interface.__name__))
        self.call = self.makeCall(call)
        self.plugins = []
        for p in plugins:
            self.addPlugin(p, call)

    def __call__(self, *arg, **kw):
        return self.call(*arg, **kw)

    def addPlugin(self, plugin, call):
        """Add plugin to my list of plugins to call, if it has the attribute
        I'm bound to.
        """
        meth = getattr(plugin, call, None)
        if meth is not None:
            if call == 'loadTestsFromModule' and \
                    len(inspect.getargspec(meth)[0]) == 2:
                orig_meth = meth
                meth = lambda module, path, **kwargs: orig_meth(module)
            self.plugins.append((plugin, meth))

    def makeCall(self, call):
        if call == 'loadTestsFromNames':
            # special case -- load tests from names behaves somewhat differently
            # from other chainable calls, because plugins return a tuple, only
            # part of which can be chained to the next plugin.
            return self._loadTestsFromNames

        meth = self.method
        if getattr(meth, 'generative', False):
            # call all plugins and yield a flattened iterator of their results
            return lambda *arg, **kw: list(self.generate(*arg, **kw))
        elif getattr(meth, 'chainable', False):
            return self.chain
        else:
            # return a value from the first plugin that returns non-None
            return self.simple

    def chain(self, *arg, **kw):
        """Call plugins in a chain, where the result of each plugin call is
        sent to the next plugin as input. The final output result is returned.
        """
        result = None
        # extract the static arguments (if any) from arg so they can
        # be passed to each plugin call in the chain
        static = [a for (static, a)
                  in zip(getattr(self.method, 'static_args', []), arg)
                  if static]
        for p, meth in self.plugins:
            result = meth(*arg, **kw)
            arg = static[:]
            arg.append(result)
        return result

    def generate(self, *arg, **kw):
        """Call all plugins, yielding each item in each non-None result.
        """
        for p, meth in self.plugins:
            result = None
            try:
                result = meth(*arg, **kw)
                if result is not None:
                    for r in result:
                        yield r
            except (KeyboardInterrupt, SystemExit):
                raise
            except:
                exc = sys.exc_info()
                yield Failure(*exc)
                continue

    def simple(self, *arg, **kw):
        """Call all plugins, returning the first non-None result.
        """
        for p, meth in self.plugins:
            result = meth(*arg, **kw)
            if result is not None:
                return result

    def _loadTestsFromNames(self, names, module=None):
        """Chainable but not quite normal. Plugins return a tuple of
        (tests, names) after processing the names. The tests are added
        to a suite that is accumulated throughout the full call, while
        names are input for the next plugin in the chain.
        """
        suite = []
        for p, meth in self.plugins:
            result = meth(names, module=module)
            if result is not None:
                suite_part, names = result
                if suite_part:
                    suite.extend(suite_part)
        return suite, names


class NoPlugins(object):
    """Null Plugin manager that has no plugins."""
    interface = IPluginInterface
    def __init__(self):
        self._plugins = self.plugins = ()

    def __iter__(self):
        return ()

    def _doNothing(self, *args, **kwds):
        pass

    def _emptyIterator(self, *args, **kwds):
        return ()

    def __getattr__(self, call):
        method = getattr(self.interface, call)
        if getattr(method, "generative", False):
            return self._emptyIterator
        else:
            return self._doNothing

    def addPlugin(self, plug):
        raise NotImplementedError()

    def addPlugins(self, plugins):
        raise NotImplementedError()

    def configure(self, options, config):
        pass

    def loadPlugins(self):
        pass

    def sort(self):
        pass


class PluginManager(object):
    """Base class for plugin managers. PluginManager is intended to be
    used only with a static list of plugins. The loadPlugins() implementation
    only reloads plugins from _extraplugins to prevent those from being
    overridden by a subclass.

    The basic functionality of a plugin manager is to proxy all unknown
    attributes through a ``PluginProxy`` to a list of plugins.

    Note that the list of plugins *may not* be changed after the first plugin
    call.
    """
    proxyClass = PluginProxy

    def __init__(self, plugins=(), proxyClass=None):
        self._plugins = []
        self._extraplugins = ()
        self._proxies = {}
        if plugins:
            self.addPlugins(plugins)
        if proxyClass is not None:
            self.proxyClass = proxyClass

    def __getattr__(self, call):
        try:
            return self._proxies[call]
        except KeyError:
            proxy = self.proxyClass(call, self._plugins)
            self._proxies[call] = proxy
        return proxy

    def __iter__(self):
        return iter(self.plugins)

    def addPlugin(self, plug):
        # allow, for instance, plugins loaded via entry points to
        # supplant builtin plugins.
        new_name = getattr(plug, 'name', object())
        self._plugins[:] = [p for p in self._plugins
                            if getattr(p, 'name', None) != new_name]
        self._plugins.append(plug)

    def addPlugins(self, plugins=(), extraplugins=()):
        """extraplugins are maintained in a separate list and
        re-added by loadPlugins() to prevent their being overwritten
        by plugins added by a subclass of PluginManager
        """
        self._extraplugins = extraplugins
        for plug in iterchain(plugins, extraplugins):
            self.addPlugin(plug)

    def configure(self, options, config):
        """Configure the set of plugins with the given options
        and config instance. After configuration, disabled plugins
        are removed from the plugins list.
        """
        log.debug("Configuring plugins")
        self.config = config
        cfg = PluginProxy('configure', self._plugins)
        cfg(options, config)
        enabled = [plug for plug in self._plugins if plug.enabled]
        self.plugins = enabled
        self.sort()
        log.debug("Plugins enabled: %s", enabled)

    def loadPlugins(self):
        for plug in self._extraplugins:
            self.addPlugin(plug)

    def sort(self):
        return sort_list(self._plugins, lambda x: getattr(x, 'score', 1), reverse=True)

    def _get_plugins(self):
        return self._plugins

    def _set_plugins(self, plugins):
        self._plugins = []
        self.addPlugins(plugins)

    plugins = property(_get_plugins, _set_plugins, None,
                       """Access the list of plugins managed by
                       this plugin manager""")


class ZeroNinePlugin:
    """Proxy for 0.9 plugins, adapts 0.10 calls to 0.9 standard.
    """
    def __init__(self, plugin):
        self.plugin = plugin

    def options(self, parser, env=os.environ):
        self.plugin.add_options(parser, env)

    def addError(self, test, err):
        if not hasattr(self.plugin, 'addError'):
            return
        # switch off to addSkip, addDeprecated if those types
        from nose.exc import SkipTest, DeprecatedTest
        ec, ev, tb = err
        if issubclass(ec, SkipTest):
            if not hasattr(self.plugin, 'addSkip'):
                return
            return self.plugin.addSkip(test.test)
        elif issubclass(ec, DeprecatedTest):
            if not hasattr(self.plugin, 'addDeprecated'):
                return
            return self.plugin.addDeprecated(test.test)
        # add capt
        capt = test.capturedOutput
        return self.plugin.addError(test.test, err, capt)

    def loadTestsFromFile(self, filename):
        if hasattr(self.plugin, 'loadTestsFromPath'):
            return self.plugin.loadTestsFromPath(filename)

    def addFailure(self, test, err):
        if not hasattr(self.plugin, 'addFailure'):
            return
        # add capt and tbinfo
        capt = test.capturedOutput
        tbinfo = test.tbinfo
        return self.plugin.addFailure(test.test, err, capt, tbinfo)

    def addSuccess(self, test):
        if not hasattr(self.plugin, 'addSuccess'):
            return
        capt = test.capturedOutput
        self.plugin.addSuccess(test.test, capt)

    def startTest(self, test):
        if not hasattr(self.plugin, 'startTest'):
            return
        return self.plugin.startTest(test.test)

    def stopTest(self, test):
        if not hasattr(self.plugin, 'stopTest'):
            return
        return self.plugin.stopTest(test.test)

    def __getattr__(self, val):
        return getattr(self.plugin, val)


class EntryPointPluginManager(PluginManager):
    """Plugin manager that loads plugins from the `nose.plugins` and
    `nose.plugins.0.10` entry points.
    """
    entry_points = (('nose.plugins.0.10', None),
                    ('nose.plugins', ZeroNinePlugin))

    def loadPlugins(self):
        """Load plugins by iterating the `nose.plugins` entry point.
        """
        from pkg_resources import iter_entry_points
        loaded = {}
        for entry_point, adapt in self.entry_points:
            for ep in iter_entry_points(entry_point):
                if ep.name in loaded:
                    continue
                loaded[ep.name] = True
                log.debug('%s load plugin %s', self.__class__.__name__, ep)
                try:
                    plugcls = ep.load()
                except KeyboardInterrupt:
                    raise
                except Exception as e:
                    # never want a plugin load to kill the test run
                    # but we can't log here because the logger is not yet
                    # configured
                    warn("Unable to load plugin %s: %s" % (ep, e),
                         RuntimeWarning)
                    continue
                if adapt:
                    plug = adapt(plugcls())
                else:
                    plug = plugcls()
                self.addPlugin(plug)
        super(EntryPointPluginManager, self).loadPlugins()


class BuiltinPluginManager(PluginManager):
    """Plugin manager that loads plugins from the list in
    `nose.plugins.builtin`.
    """
    def loadPlugins(self):
        """Load plugins in nose.plugins.builtin
        """
        from nose.plugins import builtin
        for plug in builtin.plugins:
            self.addPlugin(plug())
        super(BuiltinPluginManager, self).loadPlugins()

try:
    import pkg_resources
    class DefaultPluginManager(EntryPointPluginManager, BuiltinPluginManager):
        pass

except ImportError:
    class DefaultPluginManager(BuiltinPluginManager):
        pass

class RestrictedPluginManager(DefaultPluginManager):
    """Plugin manager that restricts the plugin list to those not
    excluded by a list of exclude methods. Any plugin that implements
    an excluded method will be removed from the manager's plugin list
    after plugins are loaded.
    """
    def __init__(self, plugins=(), exclude=(), load=True):
        DefaultPluginManager.__init__(self, plugins)
        self.load = load
        self.exclude = exclude
        self.excluded = []
        self._excludedOpts = None

    def excludedOption(self, name):
        if self._excludedOpts is None:
            from optparse import OptionParser
            self._excludedOpts = OptionParser(add_help_option=False)
            for plugin in self.excluded:
                plugin.options(self._excludedOpts, env={})
        return self._excludedOpts.get_option('--' + name)

    def loadPlugins(self):
        if self.load:
            DefaultPluginManager.loadPlugins(self)
        allow = []
        for plugin in self.plugins:
            ok = True
            for method in self.exclude:
                if hasattr(plugin, method):
                    ok = False
                    self.excluded.append(plugin)
                    break
            if ok:
                allow.append(plugin)
        self.plugins = allow



# -*- coding: utf-8 -*-
#   Copyright (C) 2021 Rocky Bernstein
#

import os

# Our local modules
from trepan.processor.command.base_subcmd import DebuggerSubcommand


class SetTempdir(DebuggerSubcommand):
    """**set tempdir** *directory*

    Set the temporary directory for temporary decompiled python files.

    This is sometimes useful remote debugging where you might set up a
    common shared location available between the debugged process and
    the front end client.

    Examples:
    ---------

        set tempdir /code/tmp  # /code is a shared directory

    See also:
    --------

    `show tempdir`"""

    in_list = True
    min_abbrev = len("temp")
    min_args = 1
    max_args = 1
    short_help = "Set a directory for storing decompiled Python"

    def run(self, args):
        dirpath = args[0]
        if os.path.isdir(dirpath):
            self.debugger.settings[self.name] = dirpath
        else:
            self.errmsg("set tempdir: directory %s not found; not changed." % dirpath)
        return

    pass


# if __name__ == '__main__':
#     from trepan.processor.command.set.tempdir import __demo_helper__ as Mhelper
#     sub = Mhelper.demo_run(SetTempdir)
#     d = sub.proc.debugger
#     sub.run(['tempdir'])
#     print(d.settings['tempdir'])
#     pass



#!/usr/bin/env python3
"Unit test for trepan.bytecode"
import inspect
import unittest

from xdis import IS_PYPY, PYTHON_VERSION_TRIPLE

from trepan.lib import bytecode as Mcode


class TestByteCode(unittest.TestCase):
    def test_contains_make_function(self):
        def sqr(x):
            return x * x

        frame = inspect.currentframe()
        co = frame.f_code
        lineno = frame.f_lineno
        self.assertFalse(Mcode.stmt_contains_opcode(co, lineno, "MAKE_FUNCTION"))
        return

    def test_op_at_frame(self):
        frame = inspect.currentframe()
        if IS_PYPY or PYTHON_VERSION_TRIPLE >= (3, 7):
            call_opcode = "CALL_METHOD"
        else:
            call_opcode = "CALL_FUNCTION"

        self.assertEqual(call_opcode, Mcode.op_at_frame(frame))
        return

    def test_is_def_frame(self):
        # Not a "def" statement because frame is wrong spot
        frame = inspect.currentframe()
        self.assertFalse(Mcode.is_def_stmt("foo(): pass", frame))
        return


if __name__ == "__main__":
    unittest.main()



# -*- coding: utf-8 -*-
#   Copyright (C) 2008-2010, 2013, 2015, 2017-2018, 2020-2021 Rocky Bernstein <rocky@gnu.org>
#
#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.
""" Functions for working with Python frames"""

import inspect
import linecache
import os
import os.path as osp
import re

import xdis
from xdis import IS_PYPY, get_opcode
from xdis.version_info import PYTHON_VERSION_TRIPLE

import trepan.lib.bytecode as Mbytecode
import trepan.lib.format as Mformat
import trepan.lib.pp as Mpp
import trepan.lib.printing as Mprint
from trepan.lib.deparse import deparse_offset
from trepan.processor.cmdfns import deparse_fn

format_token = Mformat.format_token

_with_local_varname = re.compile(r"_\[[0-9+]\]")


def count_frames(frame, count_start=0):
    "Return a count of the number of frames"
    count = -count_start
    while frame:
        count += 1
        frame = frame.f_back
    return count


import reprlib as Mrepr

_re_pseudo_file = re.compile(r"^<.+>")


# Taken from 3.7 inspect.py. However instead of an object name, we
# start with the filename. Also, oddly getsourcefile wasn't
# stripping out __pycache__. Finally, we've adapted this
# so it works back to 3.0.
#
def getsourcefile(filename):
    """Return the filename that can be used to locate an object's source.
    Return None if no way can be identified to get the source.
    """
    all_bytecode_suffixes = (".pyc", ".pyo")
    if any(filename.endswith(s) for s in all_bytecode_suffixes):
        if osp.dirname(filename).endswith("__pycache__"):
            filename = osp.join(
                osp.dirname(osp.dirname(filename)), osp.basename(filename)
            )
        filename = osp.splitext(filename)[0] + ".py"
    elif any(filename.endswith(s) for s in [".abi3.so", ".so"]):
        return None
    if osp.exists(filename):
        return filename
    # only return a non-existent filename if the module has a PEP 302 loader
    if getattr(inspect.getmodule(object, filename), "__loader__", None) is not None:
        return filename
    # or it is in the linecache
    if filename in linecache.cache:
        return filename


def deparse_source_from_code(code):
    source_text = ""
    try:
        source_lines = deparse_fn(code).split("\n")
        source_text = ""
        for i, source_text in enumerate(source_lines):
            if len(source_text) > 0:
                break
        if len(source_lines) > 1:
            source_text += "..."
        source_text = '"%s"' % source_text
    except Exception:
        pass
    return source_text


def format_stack_entry(
    dbg_obj, frame_lineno, lprefix=": ", include_location=True, color="plain"
):
    """Format and return a stack entry gdb-style.
    Note: lprefix is not used. It is kept for compatibility.
    """
    frame, lineno = frame_lineno
    filename = frame2file(dbg_obj.core, frame)

    s = ""
    if frame.f_code.co_name:
        funcname = frame.f_code.co_name
    else:
        funcname = "<lambda>"
        pass
    s = format_token(Mformat.Function, funcname, highlight=color)

    args, varargs, varkw, local_vars = inspect.getargvalues(frame)
    if "<module>" == funcname and ([], None, None,) == (
        args,
        varargs,
        varkw,
    ):
        is_module = True
        if is_exec_stmt(frame):
            fn_name = format_token(Mformat.Function, "exec", highlight=color)
            source_text = deparse_source_from_code(frame.f_code)
            s += " %s(%s)" % (
                format_token(Mformat.Function, fn_name, highlight=color),
                source_text,
            )
        else:
            fn_name = get_call_function_name(frame)
            if fn_name:
                source_text = deparse_source_from_code(frame.f_code)
                if fn_name:
                    s += " %s(%s)" % (
                        format_token(Mformat.Function, fn_name, highlight=color),
                        source_text,
                    )
            pass
    else:
        is_module = False
        try:
            parms = inspect.formatargvalues(args, varargs, varkw, local_vars)
        except Exception:
            pass
        else:
            maxargstrsize = dbg_obj.settings["maxargstrsize"]
            if len(parms) >= maxargstrsize:
                parms = "%s...)" % parms[0:maxargstrsize]
                pass
            s += parms
        pass

    # Note: ddd can't handle wrapped stack entries (yet).
    # The 35 is hoaky though. FIXME.
    if len(s) >= 35:
        s += "\n    "

    if "__return__" in frame.f_locals:
        rv = frame.f_locals["__return__"]
        s += "->"
        s += format_token(Mformat.Return, Mrepr.repr(rv), highlight=color)
        pass

    if include_location:
        is_pseudo_file = _re_pseudo_file.match(filename)
        add_quotes_around_file = not is_pseudo_file
        if is_module:
            if filename == "<string>":
                s += " in exec"
            elif not is_exec_stmt(frame) and not is_pseudo_file:
                s += " file"
        elif s == "?()":
            if is_exec_stmt(frame):
                s = "in exec"
                # exec_str = get_exec_string(frame.f_back)
                # if exec_str != None:
                #     filename = exec_str
                #     add_quotes_around_file = False
                #     pass
                # pass
            elif not is_pseudo_file:
                s = "in file"
                pass
            pass
        elif not is_pseudo_file:
            s += " called from file"
            pass

        if add_quotes_around_file:
            filename = "'%s'" % filename
        s += " %s at line %s" % (
            format_token(Mformat.Filename, filename, highlight=color),
            format_token(Mformat.LineNumber, "%r" % lineno, highlight=color),
        )
    return s


def frame2file(core_obj, frame, canonic=True):
    if canonic:
        return core_obj.filename(core_obj.canonic_filename(frame))
    else:
        return core_obj.filename(frame.f_code.co_filename)


def frame2filesize(frame):
    if "__cached__" in frame.f_globals:
        bc_path = frame.f_globals["__cached__"]
    else:
        bc_path = None
    path = frame.f_globals["__file__"]
    source_path = getsourcefile(path)
    fs_size = os.stat(source_path).st_size
    if bc_path:
        (
            version,
            timestamp,
            magic_int,
            co,
            is_pypy,
            bc_source_size,
            sip_hash,
        ) = xdis.load_module(bc_path, fast_load=True, get_code=False)
        return fs_size, bc_source_size
    elif osp.exists(path):
        return fs_size, None
    else:
        return None, None


def check_path_with_frame(frame, path):
    my_size = os.stat(path).st_size
    fs_size, bc_size = frame2filesize(frame)
    if bc_size and bc_size != my_size:
        return False, "bytecode and local files mismatch"
    if fs_size and fs_size != my_size:
        return (
            False,
            (
                "frame file size, %d bytes, and local file size, %d bytes, on file %s mismatch"
                % (fs_size, my_size, path)
            ),
        )
    return True, None


def is_exec_stmt(frame):
    """Return True if we are looking at an exec statement"""
    return hasattr(frame, "f_back") and get_call_function_name(frame) == "exec"


import dis

opc = get_opcode(PYTHON_VERSION_TRIPLE, IS_PYPY)


def get_call_function_name(frame):
    """If f_back is looking at a call function, return
    the name for it. Otherwise return None"""
    f_back = frame.f_back
    if not f_back:
        return None
    if "CALL_FUNCTION" != Mbytecode.op_at_frame(f_back):
        return None

    co = f_back.f_code
    code = co.co_code
    # labels     = dis.findlabels(code)
    linestarts = dict(dis.findlinestarts(co))
    offset = f_back.f_lasti
    while offset >= 0:
        if offset in linestarts:
            op = code[offset]
            offset += 1
            arg = code[offset]
            # FIXME: put this code in xdis
            extended_arg = 0
            while True:
                if PYTHON_VERSION_TRIPLE >= (3, 6):
                    if op == opc.EXTENDED_ARG:
                        extended_arg += arg << 8
                        continue
                    arg = code[offset] + extended_arg
                    # FIXME: Python 3.6.0a1 is 2, for 3.6.a3 we have 1
                else:
                    if op == opc.EXTENDED_ARG:
                        extended_arg += arg << 256
                        continue
                    arg = code[offset] + code[offset + 1] * 256 + extended_arg
                break

            if arg < len(co.co_names):
                return co.co_names[arg]
            else:
                return None
        offset -= 1
        pass
    return None


def print_stack_entry(proc_obj, i_stack, color="plain", opts={}):
    frame_lineno = proc_obj.stack[len(proc_obj.stack) - i_stack - 1]
    frame, lineno = frame_lineno
    intf = proc_obj.intf[-1]
    if frame is proc_obj.curframe:
        intf.msg_nocr(format_token(Mformat.Arrow, "->", highlight=color))
    else:
        intf.msg_nocr("##")
    intf.msg(
        "%d %s"
        % (i_stack, format_stack_entry(proc_obj.debugger, frame_lineno, color=color))
    )
    if opts.get("source", False):
        filename = frame2file(proc_obj.core, frame)
        line = linecache.getline(filename, lineno, frame.f_globals)
        intf.msg(line)
        pass

    if opts.get("deparse", False):
        name = frame.f_code.co_name
        if frame.f_lasti == -1:
            last_i = 0
        else:
            last_i = frame.f_lasti
        deparsed, nodeInfo = deparse_offset(frame.f_code, name, last_i, None)
        if name == "<module>":
            name == "module"
        if nodeInfo:
            extractInfo = deparsed.extract_node_info(nodeInfo)
            intf.msg(extractInfo.selectedLine)
            intf.msg(extractInfo.markerLine)
        pass
    if opts.get("full", False):
        names = list(frame.f_locals.keys())
        for name in sorted(names):
            if _with_local_varname.match(name):
                val = frame.f_locals[name]
            else:
                val = proc_obj.getval(name, frame.f_locals)
                pass
            width = opts.get("width", 80)
            Mpp.pp(val, width, intf.msg_nocr, intf.msg, prefix="%s =" % name)
            pass

        deparsed, nodeInfo = deparse_offset(frame.f_code, name, frame.f_lasti, None)
        if name == "<module>":
            name == "module"
        if nodeInfo:
            extractInfo = deparsed.extract_node_info(nodeInfo)
            intf.msg(extractInfo.selectedLine)
            intf.msg(extractInfo.markerLine)
        pass


def print_stack_trace(proc_obj, count=None, color="plain", opts={}):
    "Print count entries of the stack trace"
    if count is None:
        n = len(proc_obj.stack)
    else:
        n = min(len(proc_obj.stack), count)
    try:
        for i in range(n):
            print_stack_entry(proc_obj, i, color=color, opts=opts)
    except KeyboardInterrupt:
        pass
    return


def print_dict(s, obj, title):
    if hasattr(obj, "__dict__"):
        d = obj.__dict__
        if isinstance(d, dict):
            keys = list(d.keys())
            if len(keys) == 0:
                s += "\n  No %s" % title
            else:
                s += "\n  %s:\n" % title
            keys.sort()
            for key in keys:
                s += "    '%s':\t%s\n" % (key, d[key])
                pass
            pass
        pass
    return s


def eval_print_obj(arg, frame, format=None, short=False):
    """Return a string representation of an object"""
    try:
        if not frame:
            # ?? Should we have set up a dummy globals
            # to have persistence?
            val = eval(arg, None, None)
        else:
            val = eval(arg, frame.f_globals, frame.f_locals)
            pass
    except Exception:
        return 'No symbol "' + arg + '" in current context.'

    return print_obj(arg, val, format, short)


def print_obj(arg, val, format=None, short=False):
    """Return a string representation of an object"""
    what = arg
    if format:
        what = format + " " + arg
        val = Mprint.printf(val, format)
        pass
    s = "%s = %s" % (what, val)
    if not short:
        s += "\n  type = %s" % type(val)
        # Try to list the members of a class.
        # Not sure if this is correct or the
        # best way to do.
        s = print_dict(s, val, "object variables")
        if hasattr(val, "__class__"):
            s = print_dict(s, val.__class__, "class variables")
            pass
        pass
    return s


# Demo stuff above
if __name__ == "__main__":

    class MockDebuggerCore:
        def canonic_filename(self, frame):
            return frame.f_code.co_filename

        def filename(self, name):
            return name

        pass

    class MockDebugger:
        def __init__(self):
            self.core = MockDebuggerCore()
            self.settings = {"maxargstrsize": 80}
            pass

        pass

    frame = inspect.currentframe()
    print(frame2filesize(frame))
    pyc_file = osp.join(
        osp.dirname(__file__), "__pycache__", osp.basename(__file__)[:-3] + ".pyc"
    )
    print(pyc_file, getsourcefile(pyc_file))

    # m = MockDebugger()
    # print(format_stack_entry(m, (frame, 10,)))
    # print(format_stack_entry(m, (frame, 10,), color="dark"))
    # print("frame count: %d" % count_frames(frame))
    # print("frame count: %d" % count_frames(frame.f_back))
    # print("frame count: %d" % count_frames(frame, 1))
    # print("def statement: x=5?: %s" % repr(Mbytecode.is_def_stmt("x=5", frame)))
    # # Not a "def" statement because frame is wrong spot
    # print(Mbytecode.is_def_stmt("def foo():", frame))

    # def sqr(x):
    #     x * x

    # def fn(x):
    #     frame = inspect.currentframe()
    #     print(get_call_function_name(frame))
    #     return

    # print("=" * 30)
    # eval("fn(5)")
    # print("=" * 30)
    # print(print_obj("fn", fn))
    # print("=" * 30)
    # print(print_obj("len", len))
    # print("=" * 30)
    # print(print_obj("MockDebugger", MockDebugger))
    pass



# -*- coding: utf-8 -*-
#  Copyright (C) 2015-2018, 2020-2021 Rocky Bernstein
#
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.

import sys

if sys.version_info[:2] < (3, 9):
    if sys.version_info[:2] >= (3, 7):
        try:
            from decompyle3.semantics.fragments import code_deparse
        except ImportError:
            from uncompyle6.semantics.fragments import code_deparse
    else:
        from uncompyle6.semantics.fragments import code_deparse

from getopt import GetoptError, getopt

from pyficache import getlines, highlight_string

from trepan.lib.deparse import deparse_and_cache, deparse_offset

# Our local modules
from trepan.processor.command.base_cmd import DebuggerCommand


class DeparseCommand(DebuggerCommand):
    """**deparse** [options] [ . ]

    Options:
    ------

        -p | --parent        show parent node
        -t | --tree          show parse tree
        -o | --offset [num]  show deparse of offset NUM
        -h | --help          give this help

    deparse around where the program is currently stopped. If no offset is given,
    we use the current frame offset. If `-p` is given, include parent information.

    If an '.' argument is given, deparse the entire function or main
    program you are in.

    Output is colorized the same as source listing. Use `set highlight plain` to turn
    that off.

    Examples:
    --------

        deparse             # deparse current location
        deparse --parent    # deparse current location enclosing context
        deparse .           # deparse current function or main
        deparse --offset 6  # deparse starting at offset 6
        deparse --offsets   # show all exect deparsing offsets
        deparse --tree      # deparse and show parse tree

    See also:
    ---------

    `disassemble`, `list`, and `set highlight`"""

    short_help = "Deparse source via uncompyle6/decompyle3"
    DebuggerCommand.setup(locals(), category="data", max_args=10, need_stack=True)

    def print_text(self, text):
        if self.settings["highlight"] == "plain":
            self.msg(text)
            return
        opts = {"bg": self.settings["highlight"]}

        if "style" in self.settings:
            opts["style"] = self.settings["style"]
        self.msg(highlight_string(text, opts).strip("\n"))

    def run(self, args):
        if sys.version_info[:2] > (3, 8):
            self.errmsg("Deparsing for Python greater than 3.8 not implemented")
            return
        co = self.proc.curframe.f_code
        name = co.co_name

        try:
            opts, args = getopt(
                args[1:],
                "hpPto:O",
                ["help", "parent", "pretty", "tree", "offset=", "offsets"],
            )
        except GetoptError as err:
            # print help information and exit:
            self.errmsg(
                str(err)
            )  # will print something like "option -a not recognized"
            return

        show_parent = False
        show_tree = False
        offset = None
        show_offsets = False
        for o, a in opts:
            if o in ("-h", "--help"):
                self.proc.commands["help"].run(["help", "deparse"])
                return
            elif o in ("-O", "--offsets"):
                show_offsets = True
            elif o in ("-p", "--parent"):
                show_parent = True
            elif o in (
                "-t",
                "--tree",
            ):
                show_tree = True
            elif o in ("-o", "--offset"):
                offset = a
            else:
                self.errmsg("unhandled option '%s'" % o)
            pass
        pass
        nodeInfo = None

        if len(args) >= 1 and args[0] == ".":
            temp_filename, name_for_code = deparse_and_cache(
                co, self.errmsg, tempdir=self.settings["tempdir"]
            )
            if not temp_filename:
                return
            self.print_text("".join(getlines(temp_filename)))
            return
        elif show_offsets:
            deparsed = code_deparse(co)
            self.section("Offsets known:")
            m = self.columnize_commands(
                list(sorted(deparsed.offsets.keys(), key=lambda x: str(x[0])))
            )
            self.msg_nocr(m)
            return
        elif offset is not None:
            mess = (
                "The 'deparse' command when given an argument requires an"
                " instruction offset. Got: '%s'" % offset
            )
            last_i = self.proc.get_an_int(offset, mess)
            if last_i is None:
                return
        else:
            last_i = self.proc.curframe.f_lasti
            if last_i == -1:
                last_i = 0

        deparsed, nodeInfo = deparse_offset(co, name, last_i, self.errmsg)
        if not deparsed:
            return

        if nodeInfo:
            extractInfo = deparsed.extract_node_info(nodeInfo)
            parentInfo = None
            # print extractInfo
            if show_tree:
                p = deparsed.ast
                if show_parent:
                    parentInfo, p = deparsed.extract_parent_info(nodeInfo.node)
                self.msg(p)
            if extractInfo:
                self.rst_msg("*instruction:* %s" % (nodeInfo.node))
                self.print_text(extractInfo.selectedLine)
                self.msg(extractInfo.markerLine)
                if show_parent:
                    if not parentInfo:
                        parentInfo, p = deparsed.extract_parent_info(nodeInfo.node)
                    if parentInfo:
                        self.section("Contained in...")
                        self.rst_msg("\t*Grammar Symbol:* %s" % p.kind)
                        self.print_text(parentInfo.selectedLine)
                        self.msg(parentInfo.markerLine)
                    pass
                pass
            pass
        elif last_i == -1:
            if name:
                self.msg("At beginning of %s " % name)
            elif self.core.filename(None):
                self.msg("At beginning of program %s" % self.core.filename(None))
            else:
                self.msg("At beginning")
        else:
            self.errmsg(
                "haven't recorded info for offset %d. Offsets I know are:" % last_i
            )
            offsets = [
                key[1] for key in deparsed.offsets.keys() if isinstance(key[1], int)
            ]
            m = self.columnize_commands(list(sorted(offsets)))
            self.msg_nocr(m)
        return

    pass


if __name__ == "__main__":
    import sys

    from trepan import debugger

    d = debugger.Trepan()
    cp = d.core.processor
    command = DeparseCommand(d.core.processor)
    command.proc.frame = sys._getframe()
    command.proc.setup()
    command.run(["deparse", "--bad"])
    command.run(["deparse"])
    command.run(["deparse", "--help"])
    pass



#!/usr/bin/env python3
'Unit test for trepan.misc'
import unittest

from trepan import misc as Mmisc


class TestMisc(unittest.TestCase):

    def test_option_set(self):
        TEST_OPTS = {'a': True, 'b': 5, 'c': None}
        get_option = lambda key: Mmisc.option_set(opts, key, TEST_OPTS)
        opts={'d': 6, 'a': False}
        for opt, expect in [('a', False), 
                            ('b', 5),
                            ('c', None),
                            ('d', 6)]:
            self.assertEqual(expect, get_option(opt))
        opts=None
        for opt, expect in [('a', True), 
                            ('b', 5),
                            ('c', None),
                            ('d', None)]:
            self.assertEqual(expect, get_option(opt))
        pass

    def test_wrapped_msg(self):
        """Test misc.wrapped_lines()"""
        self.assertEqual('hi there', Mmisc.wrapped_lines('hi', 'there', 80))
        self.assertEqual('hi\n\tthere', Mmisc.wrapped_lines('hi', 'there', 5))
        return
    
    pass

if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
#   Copyright (C) 2009, 2013, 2015 Rocky Bernstein <rocky@gnu.org>
#
#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.


# Our local modules
from trepan.processor.command import base_subcmd as Mbase_subcmd


class InfoDisplay(Mbase_subcmd.DebuggerSubcommand):
    """**info display**

    Show the display expression evaluated when the program stops.

    See also:
    ---------
    `display`, `undisplay`"""

    min_abbrev = 2  # info di
    need_stack = True
    short_help = "Expressions to display when program stops"

    def run(self, args):
        lines = self.proc.display_mgr.all()
        if 0 == len(lines):
            self.errmsg("There are no auto-display expressions now.")
            return
        for line in lines:
            self.msg(line)
            pass
        return

    pass


if __name__ == "__main__":
    from trepan.processor.command import mock, info as Minfo

    d, cp = mock.dbg_setup()
    i = Minfo.InfoCommand(cp)
    sub = InfoDisplay(i)
    import inspect

    cp.curframe = inspect.currentframe()
    sub.run([])
    sub.proc.display_mgr.add(cp.curframe, "/x i")
    sub.proc.display_mgr.add(cp.curframe, "d")
    sub.run([])



#!/usr/bin/env python3
'Unit test for trepan.processor.command.list'
import os, sys  # NOQA
import unittest

from trepan import debugger
from trepan.processor.command import list as Mlist


class TestListCommand(unittest.TestCase):

    def setUp(self):
        self.listsize = 8
        self.errors = []
        self.msgs = []
        return

    def errmsg(self, msg):
        self.errors.append(msg)
        return

    def msg(self, msg):
        self.msgs.append(msg)
        return

    def print_lines(self):
        for msg in self.msgs: print(msg)
        for msg in self.errors: print(msg)

    def check_lines(self, nums):
        j = 0
        # self.print_lines()
        if len(nums) != len(self.msgs):
            self.print_lines()
            self.assertFalse(True, "len(msg): %d vs len(check): %d"
                             % (len(self.msgs), len(nums)))
            return
        for i in nums:
            self.assertEqual('%3d' % i, self.msgs[j][0:3])
            j += 1
            pass
        return

    def clear_run(self, args):
        self.msgs = []
        self.cmd.proc.current_command = ' '.join(args)
        self.cmd.run(args)

    def clear_run_check(self, args, nums):
        self.clear_run(args)
        self.check_lines(nums)
        return

    def clear_run_checksize(self, args):
        self.clear_run(args)
        self.assertEqual(self.listsize, len(self.msgs)-1)
        return

    def test_list_command(self):
        import inspect
        d               = debugger.Trepan()
        cp              = d.core.processor
        cp.curframe     = inspect.currentframe()
        cp.list_filename = cp.curframe.f_code.co_filename
        self.cmd        = Mlist.ListCommand(cp)
        self.cmd.msg    = self.msg
        self.cmd.errmsg = self.errmsg
        d.settings['listsize']  = self.listsize
        d.settings['highlight'] = 'plain'
        # from trepan.api import debug; debug()
        # Simple list command.
        self.clear_run_check(['list'], list(range(1, self.listsize+1)))
        # Check 2nd set of consecutive lines
        self.clear_run_check(['list'],
                             list(range(self.listsize+1, (2*self.listsize)+1)))
        # Try going backwards.
        self.clear_run_check(['list', '-'], list(range(1, 1 + self.listsize)))
        # And again. Since we hit the beginning it's the same as before
        self.clear_run_check(['list', '-'], list(range(1, 1 + self.listsize)))

        # BUG Simple arithmetic expression
        # self.clear_run_check(['list', '4+1'], range(4+1, 4+1+listsize))

        # List first last
        # self.clear_run_check(['list', '10', ',', '20'], list(range(10, 21)))
        # List first count
        # self.clear_run_check(['list', '10', ',',  '5'], list(range(10, 16)))

        # Module
        # BUG? without '1' below the default starts with self.listsize+1
        self.clear_run_check(['os.path', '1'], list(range(1, self.listsize+2)))

        # # Function
        # self.clear_run_checksize(['list', 'os.path.join()'])
        # self.clear_run_checksize(['list', 'self.setUp()'])

        def foo(): pass
        self.clear_run_checksize(['list', 'foo()'])

        # BUG
        # self.clear_run_check(['os.path:1'], range(1, self.listsize+1))
        # self.clear_run_check(['os.path', '10', ',5'], list(range(10, 16)))
        # Use a file name

        # if 'APPVEYOR' not in os.environ:
        #     self.clear_run_check(['list', __file__+':3', ',4'], list(range(3, 5)))

        # BUGS - but possibly the windowing thing is happening?
        # self.clear_run_check(['list', __file__, '3'], list(range(3, 5)))
        # self.clear_run_check(['list', __file__, '20', '4'], list(range(20, 24)))
        # self.clear_run_check(['list', __file__, '3', '4'], list(range(3, 5)))
        return

if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
import os, signal, unittest
from fn_helper import strarray_setup, compare_output, get_lineno


class TestSigHandler(unittest.TestCase):

    def test_handle(self):

        # FIXME!
        self.assertTrue(True)
        return

        # See that we handle a USR1 signal with a 'stop' action
        cmds = ['handle usr1 stop nopass', 'continue', 'where', 'continue']
        d = strarray_setup(cmds)

        def signal_handler(num, f):
            print('signal %d received' % num)
            return

        signal.signal(signal.SIGUSR1, signal_handler)
        d.core.start()
        ##############################
        x = 5
        os.kill(os.getpid(), signal.SIGUSR1)
        y = 6                                 # +1
        ##############################        # +2
        d.core.stop()                         # +3
        kill_line = get_lineno() - 4          # +4
        out = ['-- x = 5',
               ('''?! os.kill(os.getpid(), signal.SIGUSR1)
     called from file 'test-sig.py' at line %d''' % 
                kill_line)]
        compare_output(self, out, d, cmds)

        # Now define a signal handler and see that this is skill okay
        # "ignore" is the same as "nopass"
        # FIXME: add this common code to the helper.
        cmds = ['handle usr1 stop ignore', 'continue', 'where', 'continue']
        d.intf[-1].output.output = ['']
        d.core.step_ignore = 0
        d.intf[-1].input.input = cmds 

        def signal_handler2(num, f):
            print('signal %d received' % num)
            return
        signal.signal(signal.SIGUSR1, signal_handler2)

        d.core.start()
        ##############################
        x = 7
        os.kill(os.getpid(), signal.SIGUSR1)
        y = 8                                 # +1
        ##############################        # +2
        d.core.stop()                         # +3
        kill_line = get_lineno() - 4          # +4
        out = ['-- x = 7',
               ('''?! os.kill(os.getpid(), signal.SIGUSR1)
     called from file 'test-sig.py' at line %d''' % 
                kill_line)]
        compare_output(self, out, d, cmds)


        # How about USR2 signal with 'ignore' and 'noprint' actions?
        cmds = ['handle usr2 ignore nostop noprint', 
                'continue', 'info signal usr2', 'continue']
        d.intf[-1].output.output = ['']
        d.core.step_ignore = 0
        d.intf[-1].input.input = cmds 

        def signal_handler3(num, f):
            print('signal %d received' % num)
            return
        signal.signal(signal.SIGUSR2, signal_handler2)

        d.core.start()
        ##############################
        x = 9  # NOQA
        os.kill(os.getpid(), signal.SIGUSR2)
        y = 10  # NOQA
        ##############################
        d.core.stop()
        out = ['-- x = 9']
        compare_output(self, out, d, cmds)

        return

    pass

if __name__ == '__main__':
    unittest.main()
    pass



# -*- coding: utf-8 -*-
#   Copyright (C) 2009-2010, 2013, 2015, 2020 Rocky Bernstein
#
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.

from trepan.processor.command.base_cmd import DebuggerCommand
from trepan.lib import complete as Mcomplete


class UndisplayCommand(DebuggerCommand):
    """**undisplay** *display-number*...

    Cancel some expressions to be displayed when program stops.
    Arguments are the code numbers of the expressions to stop displaying.

    No argument cancels all automatic-display expressions and is
    the same as `delete display`.

    See also:
    ---------

    `info display` to see current list of code numbers."""

    aliases = ("und",)
    short_help = "Cancel some expressions to be displayed " "when program stops"

    DebuggerCommand.setup(locals(), category="data", min_args=1)

    def complete(self, prefix):
        completions = [str(disp.number) for disp in self.proc.display_mgr.list]
        return Mcomplete.complete_token(completions, prefix)

    def run(self, args):

        if len(args) == 1:
            self.proc.display_mgr.clear()
            return
        for i in args[1:]:
            i = self.proc.get_an_int(i, "%r must be a display number" % i)
            if i is not None:
                if not self.proc.display_mgr.delete_index(i):
                    self.errmsg("No display number %d." % i)
                    return
                pass
            pass
        return False

    pass


if __name__ == "__main__":
    from trepan import debugger as Mdebugger
    from trepan.processor import cmdproc as Mcmdproc
    import inspect

    d = Mdebugger.Trepan()
    cp = d.core.processor
    command = UndisplayCommand(d.core.processor)
    cp.curframe = inspect.currentframe()
    cp.stack, cp.curindex = Mcmdproc.get_stack(cp.curframe, None, None, cp)
    command.run(["undisplay", "z"])
    command.run(["undisplay", "1", "10"])
    pass



# -*- coding: utf-8 -*-
#
#   Copyright (C) 2008-2010, 2013-2016, 2023 Rocky Bernstein <rocky@gnu.org>
#
#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.
import inspect
import linecache
import sys
import traceback
from reprlib import Repr

import pyficache

from trepan import exception as Mexcept, misc as Mmisc, vprocessor as Mprocessor
from trepan.bwprocessor import location as Mlocation, msg as Mmsg
from trepan.lib import bytecode as Mbytecode, display as Mdisplay, thred as Mthread


def get_stack(f, t, botframe, proc_obj=None):
    """Return a stack of frames which the debugger will use for in
    showing backtraces and in frame switching. As such various frame
    that are really around may be excluded unless we are debugging the
    sebugger. Also we will add traceback frame on top if that
    exists."""
    exclude_frame = lambda f: False
    if proc_obj:
        settings = proc_obj.debugger.settings
        if not settings["dbg_trepan"]:
            exclude_frame = lambda f: proc_obj.core.ignore_filter.is_included(f)
            pass
        pass
    stack = []
    if t and t.tb_frame is f:
        t = t.tb_next
    while f is not None:
        if exclude_frame(f):
            break  # See commented alternative below
        stack.append((f, f.f_lineno))
        # bdb has:
        # if f is botframe: break
        f = f.f_back
        pass
    stack.reverse()
    i = max(0, len(stack) - 1)
    while t is not None:
        stack.append((t.tb_frame, t.tb_lineno))
        t = t.tb_next
        pass
    return stack, i


def run_hooks(obj, hooks, *args):
    """Run each function in `hooks' with args"""
    for hook in hooks:
        if hook(obj, *args):
            return True
        pass
    return False


def resolve_name(obj, command_name):
    if command_name not in obj.commands:
        return None
    return command_name


# Default settings for command processor method call
DEFAULT_PROC_OPTS = {
    # A list of debugger initialization files to read on first command
    # loop entry.  Often this something like [~/.trepanrc] which the
    # front-end sets.
    "initfile_list": []
}


class BWProcessor(Mprocessor.Processor):
    def __init__(self, core_obj, opts=None):
        Mprocessor.Processor.__init__(self, core_obj)

        self.response = {"errs": [], "msg": []}
        self.continue_running = False  # True if we should leave command loop

        self.cmd_instances = self._populate_commands()

        # command name before alias or macro resolution
        self.cmd_name = ""

        # Current command getting run
        self.current_command = ""

        self.debug_nest = 1
        self.display_mgr = Mdisplay.DisplayMgr()
        self.intf = core_obj.debugger.intf
        self.last_command = None  # Initially a no-op
        self.precmd_hooks = []

        # If not:
        # self.location         = lambda : print_location(self)

        self.preloop_hooks = []
        self.postcmd_hooks = []

        self._populate_cmd_lists()

        # Stop only if line/file is different from last time
        self.different_line = None

        # These values updated on entry. Set initial values.
        self.curframe = None
        self.event = None
        self.event_arg = None
        self.frame = None
        self.list_lineno = 0

        # Create a custom safe Repr instance and increase its maxstring.
        # The default of 30 truncates error messages too easily.
        self._repr = Repr()
        self._repr.maxstring = 100
        self._repr.maxother = 60
        self._repr.maxset = 10
        self._repr.maxfrozen = 10
        self._repr.array = 10
        self._saferepr = self._repr.repr
        self.stack = []
        self.thread_name = None
        self.frame_thread_name = None
        return

    def add_preloop_hook(self, hook, position=-1, nodups=True):
        if hook in self.preloop_hooks:
            return False
        self.preloop_hooks.insert(position, hook)
        return True

    def adjust_frame(self, pos, absolute_pos):
        """Adjust stack frame by pos positions. If absolute_pos then
        pos is an absolute number. Otherwise it is a relative number.

        A negative number indexes from the other end."""
        if not self.curframe:
            Mmsg.errmsg(self, "No stack.")
            return

        # Below we remove any negativity. At the end, pos will be
        # the new value of self.curindex.
        if absolute_pos:
            if pos >= 0:
                pos = len(self.stack) - pos - 1
            else:
                pos = -pos - 1
        else:
            pos += self.curindex

        if pos < 0:
            Mmsg.errmsg(self, "Adjusting would put us beyond the oldest frame.")
            return
        elif pos >= len(self.stack):
            Mmsg.errmsg(self, "Adjusting would put us beyond the newest frame.")
            return

        self.curindex = pos
        self.curframe = self.stack[self.curindex][0]
        self.print_location()
        self.list_lineno = None
        return

    # To be overridden in derived debuggers
    def defaultFile(self):
        """Produce a reasonable default."""
        filename = self.curframe.f_code.co_filename
        # Consider using is_exec_stmt(). I just don't understand
        # the conditions under which the below test is true.
        if filename == "<string>" and self.debugger.mainpyfile:
            filename = self.debugger.mainpyfile
            pass
        return filename

    def event_processor(self, frame, event, event_arg, prompt="Trepan"):
        "command event processor: reading a commands do something with them."
        self.frame = frame
        self.event = event
        self.event_arg = event_arg

        filename = frame.f_code.co_filename
        lineno = frame.f_lineno
        line = linecache.getline(filename, lineno, frame.f_globals)
        if not line:
            opts = {
                "output": "plain",
                "reload_on_change": self.settings("reload"),
                "strip_nl": False,
            }
            line = pyficache.getline(filename, lineno, opts)
        self.current_source_text = line
        if self.settings("skip") is not None:
            if Mbytecode.is_def_stmt(line, frame):
                return True
            if Mbytecode.is_class_def(line, frame):
                return True
            pass
        self.thread_name = Mthread.current_thread_name()
        self.frame_thread_name = self.thread_name
        self.process_commands()
        return True

    def forget(self):
        """Remove memory of state variables set in the command processor"""
        self.stack = []
        self.curindex = 0
        self.curframe = None
        self.thread_name = None
        self.frame_thread_name = None
        return

    def eval(self, arg):
        """Eval string arg in the current frame context."""
        try:
            return eval(arg, self.curframe.f_globals, self.curframe.f_locals)
        except Exception:
            t, v = sys.exc_info()[:2]
            if isinstance(t, str):
                exc_type_name = t
                pass
            else:
                exc_type_name = t.__name__
            Mmsg.errmsg(self, str("%s: %s" % (exc_type_name, arg)))
            raise
        return None  # Not reached

    def exec_line(self, line):
        if self.curframe:
            local_vars = self.curframe.f_locals
            global_vars = self.curframe.f_globals
        else:
            local_vars = None
            # FIXME: should probably have place where the
            # user can store variables inside the debug session.
            # The setup for this should be elsewhere. Possibly
            # in interaction.
            global_vars = None
        try:
            code = compile(line + "\n", '"%s"' % line, "single")
            exec(code, global_vars, local_vars)
        except Exception:
            t, v = sys.exc_info()[:2]
            if isinstance(t, bytes):
                exc_type_name = t
            else:
                exc_type_name = t.__name__
            Mmsg.errmsg(self, "%s: %s" % (str(exc_type_name), str(v)))
            pass
        return

    def ok_for_running(self, cmd_obj, name, cmd_hash):
        """We separate some of the common debugger command checks here:
        whether it makes sense to run the command in this execution state,
        if the command has the right number of arguments and so on.
        """
        if hasattr(cmd_obj, "execution_set"):
            if self.core.execution_status not in cmd_obj.execution_set:
                part1 = "Command '%s' is not available for execution " "status:" % name
                Mmsg.errmsg(
                    self,
                    Mmisc.wrapped_lines(
                        part1,
                        self.core.execution_status,
                        self.debugger.settings["width"],
                    ),
                )
                return False
            pass
        if self.frame is None and cmd_obj.need_stack:
            self.intf[-1].errmsg("Command '%s' needs an execution stack." % name)
            return False
        return True

    def process_commands(self):
        """Handle debugger commands."""
        if self.core.execution_status != "No program":
            self.setup()
            Mlocation.print_location(self, self.event)
            pass
        leave_loop = run_hooks(self, self.preloop_hooks)
        self.continue_running = False

        while not leave_loop:
            try:
                run_hooks(self, self.precmd_hooks)
                # bdb had a True return to leave loop.
                # A more straight-forward way is to set
                # instance variable self.continue_running.
                leave_loop = self.process_command()
                if leave_loop or self.continue_running:
                    break
            except EOFError:
                # If we have stacked interfaces, pop to the next
                # one.  If this is the last one however, we'll
                # just stick with that.  FIXME: Possibly we should
                # check to see if we are interactive.  and not
                # leave if that's the case. Is this the right
                # thing?  investigate and fix.
                if len(self.debugger.intf) > 1:
                    del self.debugger.intf[-1]
                    self.last_command = ""
                else:
                    if self.debugger.intf[-1].output:
                        self.debugger.intf[-1].output.writeline("Leaving")
                        raise Mexcept.DebuggerQuit
                        pass
                    break
                pass
            pass
        return run_hooks(self, self.postcmd_hooks)

    def process_command(self):
        # process command
        self.response = {"errs": [], "msg": []}
        cmd_hash = self.intf[-1].read_command()

        # FIXME: put this into a routine
        if not isinstance(cmd_hash, dict):
            Mmsg.errmsg(
                self,
                "invalid input, expecting a hash: %s" % cmd_hash,
                {"set_name": True},
            )
            self.intf[-1].msg(self.response)
            return False
        if "command" not in cmd_hash:
            Mmsg.errmsg(
                self,
                "invalid input, expecting a 'command' key: %s" % cmd_hash,
                {"set_name": True},
            )
            self.intf[-1].msg(self.response)
            return False

        self.cmd_name = cmd_hash["command"]
        cmd_name = resolve_name(self, self.cmd_name)
        if cmd_name:
            cmd_obj = self.commands[cmd_name]
            if self.ok_for_running(cmd_obj, cmd_name, cmd_hash):
                try:
                    self.response["name"] = cmd_name
                    result = cmd_obj.run(cmd_hash)
                    self.intf[-1].msg(self.response)
                    if result:
                        return result
                except (Mexcept.DebuggerQuit, Mexcept.DebuggerRestart, SystemExit):
                    # Let these exceptions propagate through
                    raise
                except Exception:
                    Mmsg.errmsg(self, "INTERNAL ERROR: " + traceback.format_exc())
                    pass
                pass
            else:
                self.undefined_cmd(cmd_name)
                pass
            pass
        return False

    def remove_preloop_hook(self, hook):
        try:
            position = self.preloop_hooks.index(hook)
        except ValueError:
            return False
        del self.preloop_hooks[position]
        return True

    def setup(self):
        """Initialization done before entering the debugger-command
        loop. In particular we set up the call stack used for local
        variable lookup and frame/up/down commands.

        We return True if we should NOT enter the debugger-command
        loop."""
        self.forget()
        if self.settings("dbg_trepan"):
            self.frame = inspect.currentframe()
            pass
        if self.event in ["exception", "c_exception"]:
            exc_type, exc_value, exc_traceback = self.event_arg
        else:
            _, _, exc_traceback = (
                None,
                None,
                None,
            )  # NOQA
            pass
        if self.frame or exc_traceback:
            self.stack, self.curindex = get_stack(self.frame, exc_traceback, None, self)
            self.curframe = self.stack[self.curindex][0]
            self.thread_name = Mthread.current_thread_name()

        else:
            self.stack = self.curframe = self.botframe = None
            pass
        if self.curframe:
            self.list_lineno = max(1, inspect.getlineno(self.curframe))
        else:
            self.list_lineno = None
            pass
        # if self.execRcLines()==1: return True
        return False

    def undefined_cmd(self, cmd):
        """Error message when a command doesn't exist"""
        Mmsg.errmsg(self, 'Undefined command: "%s". Try "help".' % cmd)
        return

    def _populate_commands(self):
        """Create an instance of each of the debugger
        commands. Commands are found by importing files in the
        directory 'command'. Some files are excluded via an array set
        in __init__.  For each of the remaining files, we import them
        and scan for class names inside those files and for each class
        name, we will create an instance of that class. The set of
        DebuggerCommand class instances form set of possible debugger
        commands."""
        cmd_instances = []
        from trepan.bwprocessor import command as Mcommand

        eval_cmd_template = "command_mod.%s(self)"
        for mod_name in Mcommand.__modules__:
            import_name = "command." + mod_name
            try:
                command_mod = getattr(__import__(import_name), mod_name)
            except Exception:
                print("Error importing %s: %s" % (mod_name, sys.exc_info()[0]))
                continue

            classnames = [
                tup[0]
                for tup in inspect.getmembers(command_mod, inspect.isclass)
                if ("DebuggerCommand" != tup[0] and tup[0].endswith("Command"))
            ]
            for classname in classnames:
                eval_cmd = eval_cmd_template % classname
                try:
                    instance = eval(eval_cmd)
                    cmd_instances.append(instance)
                except Exception:
                    print(
                        "Error loading %s from %s: %s"
                        % (classname, mod_name, sys.exc_info()[0])
                    )
                    pass
                pass
            pass
        return cmd_instances

    def _populate_cmd_lists(self):
        """Populate self.commands"""
        self.commands = {}
        for cmd_instance in self.cmd_instances:
            cmd_name = cmd_instance.name
            self.commands[cmd_name] = cmd_instance
            pass
        return

    pass


# Demo it
if __name__ == "__main__":
    from trepan.interfaces import bullwinkle as Mbullwinkle

    class Debugger:
        def __init__(self):
            self.intf = [Mbullwinkle.BWInterface()]
            self.settings = {"dbg_trepan": True, "reload": False}

        pass

    class MockCore:
        def filename(self, fn):
            return fn

        def canonic_filename(self, frame):
            return frame.f_code.co_filename

        def __init__(self):
            self.debugger = Debugger()
            return

        pass

    core = MockCore()
    bwproc = BWProcessor(core)
    print("commands:")
    commands = list(bwproc.commands.keys())
    commands.sort()
    print(commands)
    print(resolve_name(bwproc, "quit"))
    # print('-' * 10)
    # print_source_line(sys.stdout.write, 100, 'source_line_test.py')
    # print('-' * 10)
    bwproc.frame = sys._getframe()
    bwproc.setup()
    # print()
    # print('-' * 10)
    Mlocation.print_location(bwproc)

    # print 'Removing non-existing quit hook: ', bwproc.remove_preloop_hook(fn)
    # bwproc.add_preloop_hook(fn)
    # print bwproc.preloop_hooks
    # print 'Removed existing quit hook: ', bwproc.remove_preloop_hook(fn)
    pass



# -*- coding: utf-8 -*-
#   Copyright (C) 2008-2009, 2013, 2015, 2017, 2020 Rocky Bernstein <rocky@gnu.org>
#
#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.

# Our local modules
from trepan.processor.command.base_subcmd import DebuggerSubcommand
from trepan.misc import wrapped_lines


class InfoProgram(DebuggerSubcommand):
    """**info program**

    Execution status of the program. Listed are:

    * Program name

    * Instruction PC

    * Reason the program is stopped.

    See also:
    ---------

    `info line`, `info args`, `info frame`, `info pc`"""

    min_abbrev = 2  # Need at least "info pr"
    max_args = 0
    need_stack = True
    short_help = "Execution status of the program"

    def run(self, args):
        """Execution status of the program."""
        mainfile = self.core.filename(None)
        proc = self.proc
        if self.core.is_running():
            if mainfile:
                part1 = "Python program '%s' is stopped." % mainfile
            else:
                part1 = "Program is stopped."
                pass
            if proc.event:
                msg = "via a %s event." % proc.event
            else:
                msg = "."
            self.msg(part1)
            if proc.curframe:
                self.msg("PC offset is %d." % proc.curframe.f_lasti)

            if proc.event == "return":
                val = proc.event_arg
                part1 = "Return value is"
                self.msg(
                    wrapped_lines(part1, proc._saferepr(val), self.settings["width"])
                )
                pass
            elif proc.event == "call":
                proc.commands["info"].run(["info", "locals"])
            elif proc.event == "exception":
                exc_type, exc_value, exc_tb = proc.event_arg
                self.msg("Exception type: %s" % proc._saferepr(exc_type))
                if exc_value:
                    self.msg("Exception value: %s" % proc._saferepr(exc_value))
                    pass
                pass
            self.msg("It stopped %s" % msg)
            if proc.event in ["signal", "exception", "c_exception"]:
                self.msg("Note: we are stopped *after* running the " "line shown.")
                pass
        else:
            if mainfile:
                part1 = "Python program '%s'" % mainfile
                msg = "is not currently running. "
                self.msg(wrapped_lines(part1, msg, self.settings["width"]))
            else:
                self.msg("No Python program is currently running.")
                pass
            self.msg(self.core.execution_status)
            pass
        return False

    pass


if __name__ == "__main__":
    from trepan.processor.command import mock, info as Minfo

    d, cp = mock.dbg_setup()
    i = Minfo.InfoCommand(cp)
    sub = InfoProgram(i)
    sub.run([])



# -*- coding: utf-8 -*-
#
#  Copyright (C) 2009, 2010, 2013, 2015, 2020-2021, 2023 Rocky Bernstein
#
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.
import importlib
import inspect
import re
import sys

from trepan.lib.complete import complete_token, complete_token_with_next
from trepan.processor.command.base_cmd import DebuggerCommand
from trepan.processor.subcmd import Subcmd


def abbrev_stringify(name, min_abbrev):
    return "(%s)%s" % (
        name[:min_abbrev],
        name[min_abbrev:],
    )


def capitalize(s):
    # "abcd" -> "Abcd"
    if s:
        return s[0].upper() + s[1:]
    else:  # empty string
        return s
    pass


class SubcommandMgr(DebuggerCommand):

    category = "status"
    min_args = 0
    max_args = None
    name = "???"  # Need to define this!
    need_stack = False

    def __init__(self, proc, name=None, base="trepan"):
        """Initialize show subcommands. Note: instance variable name
        has to be setcmds ('set' + 'cmds') for subcommand completion
        to work."""

        DebuggerCommand.__init__(self, proc)

        # Name is set in testing
        if name is None:
            name = self.__module__.split(".")[-1]
        self.__class__.name = name

        self.cmds = Subcmd(name, self)
        self.name = name
        self._load_debugger_subcommands(name, base)
        self.proc = proc

        return

    def _load_debugger_subcommands(self, name, base="trepan"):
        """Create an instance of each of the debugger
        subcommands. Commands are found by importing files in the
        directory 'name' + 'sub'. Some files are excluded via an array set
        in __init__.  For each of the remaining files, we import them
        and scan for class names inside those files and for each class
        name, we will create an instance of that class. The set of
        DebuggerCommand class instances form set of possible debugger
        commands."""

        # Initialization
        cmd_instances = []
        class_prefix = capitalize(name)  # e.g. Info, Set, or Show
        module_dir = "%s.processor.command.%s_subcmd" % (base, name)
        mod = __import__(module_dir, None, None, ["*"])
        eval_cmd_template = "command_mod.%s(self)"

        # Import, instantiate, and add classes for each of the
        # modules found in module_dir imported above.
        for module_name in mod.__modules__:
            import_name = module_dir + "." + module_name
            try:
                command_mod = importlib.import_module(import_name)
            except ImportError:
                print(
                    (
                        "Error importing name %s module %s: %s"
                        % (import_name, module_name, sys.exc_info()[0])
                    )
                )
                continue

            # Even though we tend not to do this, it is possible to
            # put more than one class into a module/file.  So look for
            # all of them.
            classnames = [
                classname
                for classname, classvalue in inspect.getmembers(
                    command_mod, inspect.isclass
                )
                if (
                    "DebuggerCommand" != classname
                    and classname.startswith(class_prefix)
                )
            ]

            for classname in classnames:
                eval_cmd = eval_cmd_template % classname
                try:
                    instance = eval(eval_cmd)
                    self.cmds.add(instance)
                except Exception:
                    print("Error eval'ing class %s" % classname)
                    pass
                pass
            pass
        return cmd_instances

    def help(self, args):
        """Give help for a command which has subcommands. This can be
        called in several ways:
            help cmd
            help cmd subcmd
            help cmd commands

        Our shtick is to give help for the overall command only if
        subcommand or 'commands' is not given. If a subcommand is given and
        found, then specific help for that is given. If 'commands' is given
        we will list the all the subcommands.
        """
        if len(args) <= 2:
            # "help cmd". Give the general help for the command part.
            doc = self.__doc__ or self.run.__doc__
            if doc:
                self.rst_msg(doc.rstrip("\n"))
            else:
                self.proc.intf[-1].errmsg(
                    "Sorry - author mess up. "
                    + "No help registered for command"
                    + self.name
                )
                pass
            return

        subcmd_name = args[2]

        if "*" == subcmd_name:
            self.section("List of subcommands for command '%s':" % self.name)
            self.msg(self.columnize_commands(self.cmds.list()))
            return

        # "help cmd subcmd". Give help specific for that subcommand.
        cmd = self.cmds.lookup(subcmd_name)
        if cmd:
            doc = cmd.__doc__ or cmd.run.__doc__
            if doc:
                self.proc.rst_msg(doc.rstrip("\n"))
            else:
                self.proc.intf[-1].errmsg(
                    "Sorry - author mess up. No help registered for "
                    "subcommand %s of command %s" % (subcmd_name, self.name)
                )
                pass
        else:
            cmds = [c for c in self.cmds.list() if re.match("^" + subcmd_name, c)]
            if cmds == []:
                self.errmsg(
                    "No %s subcommands found matching /^%s/. "
                    'Try "help".' % (self.name, subcmd_name)
                )
            else:
                self.section(
                    'Subcommand(s) of "%s" matching /^%s/:'
                    % (
                        self.name,
                        subcmd_name,
                    )
                )
                self.msg_nocr(self.columnize_commands(cmds))
                pass
            pass
        return

    # Return an Array of subcommands that can start with +arg+. If none
    # found we just return +arg+.
    # FIXME: Not used any more?
    def complete(self, prefix):
        return complete_token(self.subcmds.subcmds.keys(), prefix)

    def complete_token_with_next(self, prefix):
        # from trepan.api import debug; debug()
        result = complete_token_with_next(self.cmds.subcmds, prefix)
        return result

    def run(self, args):
        """Ooops -- the debugger author didn't redefine this run docstring."""
        if len(args) < 2:
            # We were given cmd without a subcommand; cmd is something
            # like "show", "info" or "set". Generally this means list
            # all of the subcommands.
            self.section(
                "List of %s commands (with minimum abbreviation in "
                "parenthesis):" % self.name
            )
            for subcmd_name in self.cmds.list():
                # Some commands have lots of output.
                # they are excluded here because 'in_list' is false.
                subcmd = self.cmds.subcmds[subcmd_name]
                self.summary_help(subcmd_name, subcmd)
                pass
            return False

        subcmd_prefix = args[1]
        # We were given: cmd subcmd ...
        # Run that.
        subcmd = self.cmds.lookup(subcmd_prefix)
        if subcmd:
            nargs = len(args) - 2
            if nargs < subcmd.min_args:
                self.errmsg(
                    ("Subcommand '%s %s' needs at least %d argument(s); " + "got %d.")
                    % (self.name, subcmd.name, subcmd.min_args, nargs)
                )
                return False
            if subcmd.max_args is not None and nargs > subcmd.max_args:
                self.errmsg(
                    ("Subcommand '%s %s' takes at most %d argument(s); " + "got %d.")
                    % (self.name, subcmd.name, subcmd.max_args, nargs)
                )
                return False
            return subcmd.run(args[2:])
        else:
            return self.undefined_subcmd(self.name, subcmd_prefix)
        return  # Not reached

    def summary_help(self, subcmd_name, subcmd):
        self.msg_nocr("  %-12s -- " % abbrev_stringify(subcmd_name, subcmd.min_abbrev))
        self.rst_msg(subcmd.short_help.rstrip("\n"))
        return

    pass

    def undefined_subcmd(self, cmd, subcmd):
        """Error message when subcommand asked for but doesn't exist"""
        self.proc.intf[-1].errmsg(
            ('Undefined "%s" subcommand: "%s". ' + 'Try "help %s *".')
            % (cmd, subcmd, cmd)
        )
        return

    pass


if __name__ == "__main__":
    pass



# This file is needs to be multi-lingual in both Python and POSIX
# shell which "exec()" or "source" it respectively.

# This file should define a variable __version__ which we use as the
# debugger version number.

# fmt: off
__version__="1.2.9"  # noqa



"""
This plugin installs a DEPRECATED error class for the :class:`DeprecatedTest`
exception. When :class:`DeprecatedTest` is raised, the exception will be logged
in the deprecated attribute of the result, ``D`` or ``DEPRECATED`` (verbose)
will be output, and the exception will not be counted as an error or failure.
It is enabled by default, but can be turned off by using ``--no-deprecated``.
"""

from nose.plugins.errorclass import ErrorClass, ErrorClassPlugin


class DeprecatedTest(Exception):
    """Raise this exception to mark a test as deprecated.
    """
    pass


class Deprecated(ErrorClassPlugin):
    """
    Installs a DEPRECATED error class for the DeprecatedTest exception. Enabled
    by default.
    """
    enabled = True
    deprecated = ErrorClass(DeprecatedTest,
                            label='DEPRECATED',
                            isfailure=False)

    def options(self, parser, env):
        """Register commandline options.
        """
        env_opt = 'NOSE_WITHOUT_DEPRECATED'
        parser.add_option('--no-deprecated', action='store_true',
                          dest='noDeprecated', default=env.get(env_opt, False),
                          help="Disable special handling of DeprecatedTest "
                          "exceptions.")

    def configure(self, options, conf):
        """Configure plugin.
        """
        if not self.can_configure:
            return
        self.conf = conf
        disable = getattr(options, 'noDeprecated', False)
        if disable:
            self.enabled = False



# -*- coding: utf-8 -*-
#  Copyright (C) 2009, 2013, 2015, 2018-2020 Rocky Bernstein
#
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.

from getopt import getopt, GetoptError

# Our local modules
from trepan.processor.command.base_cmd import DebuggerCommand
from trepan.lib import stack as Mstack


class BacktraceCommand(DebuggerCommand):
    """**backtrace** [*options*] [*count*]

    Print backtrace of all stack frames, or innermost *count* frames.

    With a negative argument, print outermost -*count* frames.

    An arrow indicates the 'current frame'. The current frame determines
    the context used for many debugger commands such as expression
    evaluation or source-line listing.

    *options* are:

       -d | --deparse - show deparsed call position
       -s | --source  - show source code line
       -f | --full    - locals of each frame
       -h | --help    - give this help

    Examples:
    ---------

       backtrace      # Print a full stack trace
       backtrace 2    # Print only the top two entries
       backtrace -1   # Print a stack trace except the initial (least recent) call.
       backtrace -s   # show source lines in listing
       backtrace -d   # show deparsed source lines in listing
       backtrace -f   # show with locals
       backtrace -df  # show with deparsed calls and locals
       backtrace --deparse --full   # same as above

    See also:
    ---------

    `frame`, `locals`, `global`, `deparse`, `list`."""

    aliases = ("bt", "where")
    short_help = "Print backtrace of stack frames"

    DebuggerCommand.setup(locals(), category="stack", max_args=4, need_stack=True)

    def run(self, args):

        try:
            opts, args = getopt(args[1:], "hfds", "help deparse full source".split())
        except GetoptError as err:
            # print help information and exit:
            print(str(err))  # will print something like "option -a not recognized"
            return

        bt_opts = {"width": self.settings["width"]}
        for o, a in opts:
            if o in ("-h", "--help"):
                self.proc.commands["help"].run(["help", "backtrace"])
                return
            elif o in ("-d", "--deparse"):
                bt_opts["deparse"] = True
            elif o in ("-f", "--full"):
                bt_opts["full"] = True
            elif o in ("-s", "--source"):
                bt_opts["source"] = True
            else:
                self.errmsg("unhandled option '%s'" % o)
            pass

        if len(args) > 0:
            at_most = len(self.proc.stack)
            if at_most == 0:
                self.errmsg("Stack is empty.")
                return False
            min_value = -at_most + 1
            count = self.proc.get_int(
                args[0],
                min_value=min_value,
                cmdname="backtrace",
                default=0,
                at_most=at_most,
            )
            if count is None:
                return False
            if count < 0:
                count = at_most - count
                pass
            elif 0 == count:
                count = None
        else:
            count = None
            pass

        if not self.proc.curframe:
            self.errmsg("No stack.")
            return False
        Mstack.print_stack_trace(
            self.proc, count, color=self.settings["highlight"], opts=bt_opts
        )
        return False

    pass


if __name__ == "__main__":
    from trepan.processor import cmdproc
    from trepan.debugger import Trepan

    d = Trepan()
    cp = d.core.processor
    command = BacktraceCommand(cp)
    command.run(["backtrace", "wrong", "number", "of", "args"])

    def nest_me(cp, command, i):
        import inspect

        if i > 1:
            cp.curframe = inspect.currentframe()
            cp.stack, cp.curindex = cmdproc.get_stack(cp.curframe, None, None, cp)
            print("-" * 10)
            command.run(["backtrace"])
            print("-" * 10)
            command.run(["backtrace", "1"])
        else:
            nest_me(cp, command, i + 1)
        return

    def ignore_me(cp, command, i):
        print("=" * 10)
        nest_me(cp, command, 1)
        print("=" * 10)
        cp.core.add_ignore(ignore_me)
        nest_me(cp, command, 1)
        return

    cp.forget()
    command.run(["backtrace"])
    print("-" * 10)
    ignore_me(cp, command, 1)
    command.run(["backtrace", "1"])
    print("-" * 10)
    command.run(["backtrace", "-1"])
    print("-" * 10)
    command.run(["backtrace", "3"])
    print("-" * 10)
    command.run(["backtrace", "-2"])
    print("-" * 10)
    pass



# -*- coding: utf-8 -*-
#  Copyright (C) 2009, 2013-2015, 2020, 2023 Rocky Bernstein
#
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.
import sys

# Our local modules
from trepan.processor.command.base_cmd import DebuggerCommand
from trepan.lib.stack import count_frames


class FinishCommand(DebuggerCommand):
    """**finish** [*level*]

    Continue execution until leaving the current function. When *level* is
    specified, that many frame levels need to be popped. Note that *yield*
    and exceptions raised my reduce the number of stack frames. Also, if a
    thread is switched, we stop ignoring levels.

    See the `break` command if you want to stop at a particular point in a

    See also:
    ---------

    `continue`, `step`, `next`."""

    # FIXME: add finish [levels|fn]
    # If fn is given, that's a short-hand way of looking up how many levels
    # until that frame. However the same provisions regarding stopping,
    # exceptions, 'yield'ing and so on still apply.

    aliases = ("fin",)
    execution_set = ["Running"]
    max_args = 1
    short_help = "Execute until selected stack frame returns"
    DebuggerCommand.setup(locals(), category="running", max_args=1, need_stack=True)

    def run(self, args):
        if self.proc.stack is None:
            return False
        if len(args) <= 1:
            levels = 1
        else:
            levels = self.proc.get_int(args[1], default=1, cmdname="finish")
            if levels is None:
                return False
            pass

        # print "+++ %d" % levels
        self.core.step_events = ["return"]
        self.core.stop_on_finish = True
        self.core.stop_level = count_frames(self.proc.frame) - levels
        self.core.last_frame = self.proc.frame
        self.proc.continue_running = True  # Break out of command read loop
        return True

    pass


if __name__ == "__main__":
    from mock import MockDebugger

    d = MockDebugger()
    cmd = FinishCommand(d.core.processor)
    # Need to have a subroutine to get at least one frame f_back.

    def demo_finish(cmd):
        for c in (
            ["finish", "1"],
            ["finish", "wrong", "number", "of", "args"],
            ["finish", "5"],
            ["finish", "0*5+1"],
        ):
            cmd.continue_running = False
            cmd.proc.stack = [
                (
                    sys._getframe(0),
                    14,
                )
            ]
            result = cmd.run(c)
            print("Execute result: %s" % result)
            print(
                "stop_frame %s, continue_running: %s"
                % (
                    cmd.core.stop_frame,
                    cmd.continue_running,
                )
            )
            pass
        return

    demo_finish(cmd)
    pass



#!/usr/bin/env python3
'Unit test for trepan.processor.command.quit'
import unittest

from trepan.processor.command import quit as Mquit

from cmdhelper import dbg_setup


class TestQuit(unittest.TestCase):
    """Tests QuitCommand class"""

    def test_quit(self):
        """Test processor.command.quit.QuitCommand.run()"""
        d, cp = dbg_setup()
        command = Mquit.QuitCommand(cp)
        try:
            command.run(['quit'])
        except:
            self.assertTrue(True)
        else:
            self.assertTrue(False)
        return

if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
import os, unittest
from fn_helper import strarray_setup, compare_output


class TestSkip(unittest.TestCase):
    @unittest.skip("FIXME: figure out why this doesn't work")
    def test_skip(self):

        # See that we can skip without parameter. (Same as 'skip 1'.)
        cmds = ['skip', 'continue']
        d = strarray_setup(cmds)
        d.core.start()
        ##############################
        x = 4
        x = 5
        y = 7  # NOQA
        ##############################
        d.core.stop()
        out = ['-- x = 4',    # x = 4 is shown in prompt, but not *run*.
               '-- x = 5']
        compare_output(self, out, d, cmds)
        self.assertEqual(5, x)  # Make sure lines were skipped.

        # See that we can skip with a count value
        print("skipping second skip test")
        return
        cmds = ['skip 2', 'continue']
        d = strarray_setup(cmds)
        d.core.start()
        ##############################
        x = 10
        x = 9
        z = 7  # NOQA
        ##############################
        d.core.stop(options={'remove': True})
        out = ['-- x = 10',     # x = 10 is shown in prompt, but not run.
               '-- z = 7']
        compare_output(self, out, d, cmds)
        self.assertEqual(5, x)  # Make sure x = 10, 9 were skipped.
        return
    pass

if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
#  Copyright (C) 2009, 2013-2015, 2020, 2023 Rocky Bernstein
#
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.

import inspect
import sys
import threading

from trepan.lib import thred as Mthread
from trepan.lib.complete import complete_token
from trepan.processor.cmdproc import get_stack

# Our local modules
from trepan.processor.command.base_cmd import DebuggerCommand
from trepan.processor.frame import adjust_frame, frame_low_high


class FrameCommand(DebuggerCommand):
    """**frame** [*thread-Name*|*thread-number*] [*frame-number*]

    Change the current frame to frame *frame-number* if specified, or the
    current frame, 0, if no frame number specified.

    If a thread name or thread number is given, change the current frame
    to a frame in that thread. Dot (.) can be used to indicate the name of
    the current frame the debugger is stopped in.

    A negative number indicates the position from the other or
    least-recently-entered end.  So `frame -1` moves to the oldest frame,
    and `frame 0` moves to the newest frame. Any variable or expression
    that evaluates to a number can be used as a position, however due to
    parsing limitations, the position expression has to be seen as a single
    blank-delimited parameter. That is, the expression `(5*3)-1` is okay
    while `(5 * 3) - 1)` isn't.

    **Examples:**

       frame     # Set current frame at the current stopping point
       frame 0   # Same as above
       frame 5-5 # Same as above. Note: no spaces allowed in expression 5-5
       frame .   # Same as above. "current thread" is explicit.
       frame . 0 # Same as above.
       frame 1   # Move to frame 1. Same as: frame 0; up
       frame -1  # The least-recent frame
       frame MainThread 0 # Switch to frame 0 of thread MainThread
       frame MainThread   # Same as above
       frame -2434343 0   # Use a thread number instead of name

    See also:
    ---------

    `up`, `down`, `backtrace`, and `info threads`."""

    short_help = "Select and print a stack frame"

    DebuggerCommand.setup(locals(), category="stack", max_args=2, need_stack=True)

    def complete(self, prefix):
        proc_obj = self.proc
        low, high = frame_low_high(proc_obj, None)
        ary = [str(low + i) for i in range(high - low + 1)]
        # FIXME: add in Thread names
        return complete_token(ary, prefix)

    def find_and_set_debugged_frame(self, frame, thread_id):
        """The dance we have to do to set debugger frame state to
        *frame*, which is in the thread with id *thread_id*. We may
        need to the hide initial debugger frames.
        """
        thread = threading._active[thread_id]
        thread_name = thread.getName()
        if (
            not self.settings["dbg_trepan"]
            and thread_name == Mthread.current_thread_name()
        ):
            # The frame we came in on ('current_thread_name') is
            # the same as the one we want to switch to. In this case
            # we need to some debugger frames are in this stack so
            # we need to remove them.
            newframe = Mthread.find_debugged_frame(frame)
            if newframe is not None:
                frame = newframe
            pass
        # FIXME: else: we might be blocked on other threads which are
        # about to go into the debugger it not for the fact this one got there
        # first. Possibly in the future we want
        # to hide the blocks into threading of that locking code as well.

        # Set stack to new frame
        self.stack, self.curindex = get_stack(frame, None, self.proc)
        self.proc.stack, self.proc.curindex = self.stack, self.curindex
        self.proc.frame_thread_name = thread_name
        return

    def one_arg_run(self, position_str):
        """The simple case: thread frame switching has been done or is
        not needed and we have an explicit position number as a string"""
        frame_num = self.proc.get_an_int(
            position_str,
            ("The 'frame' command requires a" + " frame number. Got: %s")
            % position_str,
        )
        if frame_num is None:
            return False

        i_stack = len(self.proc.stack)
        if i_stack == 0:
            self.errmsg("No frames recorded")
            return False

        if frame_num < -i_stack or frame_num > i_stack - 1:
            self.errmsg(
                ("Frame number has to be in the range %d to %d." + " Got: %d (%s).")
                % (-i_stack, i_stack - 1, frame_num, position_str)
            )
            return False
        else:
            adjust_frame(self.proc, "frame", pos=frame_num, absolute_pos=True)
            return True
        return  # Not reached

    def get_from_thread_name_or_id(self, name_or_id, report_error=True):
        """See if *name_or_id* is either a thread name or a thread id.
        The frame of that id/name is returned, or None if name_or_id is
        invalid."""
        thread_id = self.proc.get_int_noerr(name_or_id)
        if thread_id is None:
            # Must be a "frame" command with frame name, not a frame
            # number (or invalid command).
            name2id = Mthread.map_thread_names()
            if name_or_id == ".":
                name_or_id = Mthread.current_thread_name()
                pass
            thread_id = name2id.get(name_or_id)
            if thread_id is None:
                self.errmsg("I don't know about thread name %s." % name_or_id)
                return None, None
            pass
        # Above we should have set thread_id. Now see if we can
        # find it.
        threads = sys._current_frames()
        frame = threads.get(thread_id)
        if frame is None and report_error:
            self.errmsg(
                "I don't know about thread number %s (%d)." % name_or_id, thread_id
            )
            # self.info_thread_terse()
            return None, None
        return frame, thread_id

    def run(self, args):
        """Run a frame command. This routine is a little complex
        because we allow a number parameter variations."""

        if len(args) == 1:
            # Form is: "frame" which means "frame 0"
            position_str = "0"
        elif len(args) == 2:
            # Form is: "frame {position | thread}
            name_or_id = args[1]
            frame, thread_id = self.get_from_thread_name_or_id(name_or_id, False)
            if frame is None:
                # Form should be: frame position
                position_str = name_or_id
            else:
                # Form should be: "frame thread" which means
                # "frame thread 0"
                position_str = "0"
                self.find_and_set_debugged_frame(frame, thread_id)
                pass
        elif len(args) == 3:
            # Form is: frame <thread> <position>
            name_or_id = args[1]
            position_str = args[2]
            frame, thread_id = self.get_from_thread_name_or_id(name_or_id)
            if frame is None:
                # Error message was given in routine
                return
            self.find_and_set_debugged_frame(frame, thread_id)
            pass
        self.one_arg_run(position_str)
        return False


if __name__ == "__main__":
    from trepan import debugger as Mdebugger

    d = Mdebugger.Trepan()
    cp = d.core.processor
    command = FrameCommand(cp)
    command.run(["frame"])
    command.run(["frame", "1"])
    print("=" * 20)
    cp.curframe = inspect.currentframe()
    cp.stack, cp.curindex = get_stack(cp.curframe, None, None, cp)

    def showit(cmd):
        print("=" * 20)
        cmd.run(["frame"])
        print("-" * 20)
        cmd.run(["frame", "MainThread"])
        print("-" * 20)
        cmd.run(["frame", ".", "0"])
        print("-" * 20)
        cmd.run(["frame", "."])
        print("=" * 20)
        return

    # showit(command)

    class BgThread(threading.Thread):
        def __init__(self, fn, cmd):
            threading.Thread.__init__(self)
            self.fn = fn
            self.cmd = cmd
            return

        def run(self):
            self.fn(self.cmd)
            return

        pass

    pass

    background = BgThread(showit, command)
    background.start()
    background.join()  # Wait for the background task to finish
    pass



#!/usr/bin/env python3
import sys

from setuptools import find_packages, setup

SYS_VERSION = sys.version_info[0:2]
if not ((3, 1) <= SYS_VERSION <= (3, 10)):
    mess = "Python Versions 3.1 to 3.10 are supported only in this package."
    if (2, 4) <= SYS_VERSION <= (2, 7):
        mess += "\nFor your Python, version %s, See trepan2" % sys.version[0:3]
    elif SYS_VERSION < (2, 4):
        mess += "\nFor your Python, version %s, see pydb" % sys.version[0:3]
    print(mess)
    raise Exception(mess)

# Get the package information used in setup().
from __pkginfo__ import (
    __version__,
    author,
    author_email,
    classifiers,
    entry_points,
    install_requires,
    license,
    long_description,
    modname,
    py_modules,
    short_desc,
    web,
    zip_safe,
)

__import__("pkg_resources")

packages = find_packages()

setup(
    author=author,
    author_email=author_email,
    classifiers=classifiers,
    data_files=[
        (
            "trepan/processor/command/help",
            [
                "trepan/processor/command/help/arange.rst",
                "trepan/processor/command/help/command.rst",
                "trepan/processor/command/help/examples.rst",
                "trepan/processor/command/help/filename.rst",
                "trepan/processor/command/help/location.rst",
                "trepan/processor/command/help/range.rst",
                "trepan/processor/command/help/suffixes.rst",
            ],
        )
    ],
    description=short_desc,
    entry_points=entry_points,
    install_requires=install_requires,
    license=license,
    long_description=long_description,
    long_description_content_type="text/x-rst",
    name=modname,
    packages=packages,
    py_modules=py_modules,
    test_suite="nose.collector",
    url=web,
    version=__version__,
    zip_safe=zip_safe,
)



# -*- coding: utf-8 -*-
#  Copyright (C) 2013 Rocky Bernstein
#
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.
import os

# Our local modules
from trepan.bwprocessor.command import base_cmd as Mbase_cmd


class StepCommand(Mbase_cmd.DebuggerCommand):
    """
    step statements

    **Input Fields:**

       { command  => 'step',
         [count   => <integer>],
       }

    If *count* is given, that many statements will be stepped. If it
    is not given, 1 is used, i.e. stop at the next statement.

    **Output Fields:**

       { name     => 'step',
         count    => <integer>,
         [errmsg  => <error-message-array>]
         [msg     => <message-text array>]
       }"""

    name = os.path.basename(__file__).split(".")[0]
    need_stack = True

    def run(self, cmd_hash):
        if "step_count" in cmd_hash:
            step_count = cmd_hash["step_count"] - 1
        else:
            step_count = 0
            pass
        self.proc.debugger.core.step_ignore = 0
        self.core.stop_level = None
        self.core.last_frame = None
        self.core.stop_on_finish = False
        self.proc.continue_running = True  # Break out of command read loop
        self.proc.response["step_count"] = step_count + 1
        return True

    pass


if __name__ == "__main__":
    # from mock import MockDebugger
    # d = MockDebugger()
    # cmd = StepCommand(d.core.processor)
    # for c in (['s', '5'],
    #           ['step', '1+2'],
    #           ['s', 'foo']):
    #     d.core.step_ignore = 0
    #     cmd.proc.continue_running = False
    #     result = cmd.run(c)
    #     print 'Execute result: %s' % result
    #     print 'step_ignore %s' % repr(d.core.step_ignore)
    #     print 'continue_running: %s' % cmd.proc.continue_running
    #     pass
    # for c in (['s'], ['step+'], ['s-'], ['s!'], ['s>'], ['s<']):
    #     d.core.step_ignore = 0
    #     cmd.continue_running = False
    #     result = cmd.run(c)
    #     print 'different line %s:' % c[0], cmd.core.different_line
    #     pass
    pass



# -*- coding: utf-8 -*-
#   Copyright (C) 2015, 2023 Rocky Bernstein <rocky@gnu.org>
""" Location routines"""

import linecache
import tempfile

import pyficache

from trepan.lib import stack as Mstack


def format_location(proc_obj):
    """Show where we are. GUI's and front-end interfaces often
    use this to update displays. So it is helpful to make sure
    we give at least some place that's located in a file.
    """
    i_stack = proc_obj.curindex
    if i_stack is None or proc_obj.stack is None:
        return False
    location = {}
    core_obj = proc_obj.core
    dbgr_obj = proc_obj.debugger

    # Evaluation routines like "exec" don't show useful location
    # info. In these cases, we will use the position before that in
    # the stack.  Hence the looping below which in practices loops
    # once and sometimes twice.
    while i_stack >= 0:
        frame_lineno = proc_obj.stack[i_stack]
        i_stack -= 1
        frame, lineno = frame_lineno

        filename = Mstack.frame2file(core_obj, frame)

        location["filename"] = filename
        location["fn_name"] = frame.f_code.co_name
        location["lineno"] = lineno

        if "<string>" == filename and dbgr_obj.eval_string:
            filename = pyficache.unmap_file(filename)
            if "<string>" == filename:
                fd = tempfile.NamedTemporaryFile(
                    suffix=".py", prefix="eval_string", delete=False
                )
                fd.write(bytes(dbgr_obj.eval_string, "UTF-8"))
                fd.close()
                pyficache.remap_file(fd.name, "<string>")
                filename = fd.name
                pass
            pass

        opts = {"reload_on_change": proc_obj.settings("reload"), "output": "plain"}
        line = pyficache.getline(filename, lineno, opts)
        if not line:
            line = linecache.getline(filename, lineno, proc_obj.curframe.f_globals)
            pass

        if line and len(line.strip()) != 0:
            location["text"] = line
            pass
        if "<string>" != filename:
            break
        pass

    return location


def print_location(proc_obj, event=None):
    response = proc_obj.response
    response["name"] = "status"
    response["location"] = format_location(proc_obj)
    if event:
        response["event"] = event
        if event in ["return", "exception"]:
            val = proc_obj._saferepr(proc_obj.event_arg)
            event["arg"] = val
            pass
        pass
    proc_obj.intf[-1].msg(response)
    return


# Demo it
if __name__ == "__main__":

    class MockDebugger:
        def __init__(self):
            self.eval_string = None

        pass

    class MockProcessor:
        def __init__(self, core_obj):
            self.curindex = 0
            self.stack = []
            self.core = core_obj
            self.debugger = MockDebugger()
            self.opts = {"highlight": "plain", "reload": False}
            pass

        def settings(self, key):
            return self.opts[key]

        pass

    class MockCore:
        def filename(self, fn):
            return fn

        def canonic_filename(self, frame):
            return frame.f_code.co_filename

        pass

    core = MockCore()
    cmdproc = MockProcessor(core)

    import sys

    cmdproc.curframe = cmdproc.frame = sys._getframe()
    cmdproc.stack.append((sys._getframe(), 10))

    import pprint

    pp = pprint.PrettyPrinter()
    pp.pprint(format_location(cmdproc))

    def test(cmdproc, pp):
        cmdproc.stack[0:0] = [(sys._getframe(1), 1)]
        pp.pprint(format_location(cmdproc))
        pass

    eval("test(cmdproc, pp)")
    cmdproc.debugger.eval_string = "Fooled you!"
    eval("test(cmdproc, pp)")
    pass



#!/usr/bin/env python3
'Unit test for trepan.exception'
import unittest

from trepan import exception as Mexcept


class TestDeguggerExcept(unittest.TestCase):

    def test_debugger_restart(self):
        try:
            raise Mexcept.DebuggerRestart(['a', 'b'])
        except Mexcept.DebuggerRestart:
            import sys
            self.assertEqual(['a', 'b'], sys.exc_info()[1].sys_argv)
        else:
            self.assertFalse(True)
        pass
        return
        
if __name__ == '__main__':
    unittest.main()
